#include "wButton.h"


#include <QtGui>


wButton::wButton(QWidget *parent) :  QWidget(parent){
    pic = new QLabel("",this);
    pic->setMouseTracking(true);
    pic->installEventFilter(this);
    pic->setPixmap(QPixmap(":images/logo.png"));

    QFont font;
    font.setPixelSize(20);

    QBrush brush;
    

    brush.setColor(QColor(255, 255, 255, 255));
    brush.setStyle(Qt::SolidPattern);
    desc_palette1.setBrush(QPalette::Active, QPalette::WindowText, brush);
    desc_palette1.setBrush(QPalette::Inactive, QPalette::WindowText, brush);

    brush.setColor(QColor(0, 0, 0, 255));
    brush.setStyle(Qt::SolidPattern);
    desc_palette2.setBrush(QPalette::Active, QPalette::WindowText, brush);
    desc_palette2.setBrush(QPalette::Inactive, QPalette::WindowText, brush);


    brush.setColor(QColor(221, 163, 40, 255));   //#DDA328
    brush.setStyle(Qt::SolidPattern);
    option_palette1.setBrush(QPalette::Active, QPalette::WindowText, brush);
    option_palette1.setBrush(QPalette::Inactive, QPalette::WindowText, brush);

    brush.setColor(QColor(255, 255, 255, 255));   //#DDA328
    brush.setStyle(Qt::SolidPattern);
    option_palette2.setBrush(QPalette::Active, QPalette::WindowText, brush);
    option_palette2.setBrush(QPalette::Inactive, QPalette::WindowText, brush);



    option = new QLabel("",this);
    option->setFont(font);
    option->setPalette(option_palette1);
    option->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
    option->setMouseTracking(true);
    option->installEventFilter(this);


    desc = new QLabel("",this);
    desc->setFont(font);
    desc->setPalette(desc_palette1);
    desc->setAlignment(Qt::AlignLeft|Qt::AlignVCenter);
    desc->setMouseTracking(true);
    desc->installEventFilter(this);

}
void wButton::resizeEvent ( QResizeEvent * event ) {
    pic->setFixedSize(event->size().width(),event->size().height());
    option->setFixedSize(60,event->size().height());
    desc->setFixedSize(event->size().width()-60,event->size().height());
    desc->move(65,0);
}


void wButton::setText(QString text1,QString text2){
    option->setText(text1);
    desc->setText(text2);
}

void wButton::setText(QString text){
    desc->setText(text);
}


void wButton::setFont(QFont font1,QFont font2){
    option->setFont(font1);
    desc->setFont(font2);
}




void wButton::setPicEvent(QString picleave,QString picenter, QString picclick){

    if (picenter == "") picenter=picleave;
    if (picclick == "") picclick=picenter;

    picLeave=picleave;
    picEnter=picenter;
    picClick=picclick;

    setPicture(picLeave);

}


bool wButton::eventFilter(QObject *o, QEvent *e){
    if (block==1) return QWidget::eventFilter(o, e);

    QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(e);

    if (e->type() == QEvent::Leave){
        if ((o == option or o == desc or o==pic) and picLeave.length()>0){
            setPicture(picLeave);
            option->setPalette(option_palette1);
            desc->setPalette(desc_palette1);
            return 1;
        }
    }

    if (e->type() ==   QEvent::Enter){
        if ((o == option or o == desc or o==pic) and picEnter.length()>0){
            setPicture(picEnter);
            option->setPalette(option_palette2);
            desc->setPalette(desc_palette2);
            return 1;
        }
    }

    if (e->type() ==   QEvent::MouseMove ){
        if ((o == option or o == desc or o==pic) and picEnter.length()>0){
            setPicture(picEnter);
            option->setPalette(option_palette2);
            desc->setPalette(desc_palette2);
            return 1;
        }
    }


    if (e->type() == QEvent::MouseButtonPress ){
        QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(e);
        if (mouseEvent->button() == Qt::LeftButton){
            if (o == option or o == desc or o==pic){
                setPicture(picClick);
            }
        }
    }

    if (e->type() == QEvent::MouseButtonRelease){
        QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(e);
        if (mouseEvent->button() == Qt::LeftButton){
            if (o == option or o == desc or o==pic){
                setPicture(picEnter);
                emit clicked();
                return 1;
            }
        }
    }

    return QWidget::eventFilter(o, e);
}


void wButton::setPicture(QString picture){
    #ifndef __ANDROID__
    QImage im;
    QPixmap pixmap;

    pic->clearMask();
    clearMask();
    #endif

    pic->setPixmap(QPixmap(picture));

    #ifndef __ANDROID__
    im.load(picture);
    pixmap= QPixmap::fromImage(im);
    pic->setPixmap(pixmap);

    pic->setMask(pixmap.mask());
    setMask(pixmap.mask());
    #endif
}

void wButton::setBlock(int value){
    block=value;

    if (block==0){
        setPicture(picLeave);
        option->setPalette(option_palette1);
        desc->setPalette(desc_palette1);
    }
}

