#include "wGame.h"
#include <QMessageBox>
#include <QApplication>


wGame::wGame(QWidget *parent) :    QWidget(parent){
    this->setFixedSize(parent->width(),parent->height());

    // Инициализация кнопок (вопрос, и кнопки ответы, подсказки)
    initButton();

    // Инициализация звука
    initSound();


    QLabel *logo = new QLabel(this);
    logo->setPixmap(QPixmap(":images/logo.png"));
    logo->setFixedSize(473,257);
    logo->move(119,20);


    QLabel *line = new QLabel(this);
    line->setPixmap(QPixmap(":images/line.png"));
    line->setFixedSize(2,514);
    line->show();
    line->move(720,0);


    ///// Инициализация таймеров
    // таймер ожидание проверки выбора
    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(continueGame()));

    // таймер генерации следующего вопроса.
    timer_pause = new QTimer(this);
    connect(timer_pause, SIGNAL(timeout()), this, SLOT(continueGame2()));


    // Таблица Очков
    score_table = new wScore(this);
    score_table->show();
    score_table->move(730,85);

    // Табло
    tablo = new wTablo(this);
    tablo->move(50,15);
    tablo->hide();



}

void wGame::initGame(){
    // обнуление очков
    score=1;
    score_table->setScore(score);

    // востановление кнопок
    for (int i=1;i<5;i++){
        anwser[i]->setPicEvent(":images/anwser_bg.png",":images/anwser_hover_bg.png","");
        anwser[i]->setBlock(0);
    }

    // Создание вопроса
    genQuestion(score);

    // Играть фоновую мелодию
#ifndef __ANDROID__
    Mix_PlayChannel(1,sound_fon[1],-1);
#endif

    // подсказки
    help_50->setPicEvent(":images/50.png",":images/50.png");
    help_help->setPicEvent(":images/help.png",":images/help.png");
    help_phone->setPicEvent(":images/phone.png",":images/phone.png");

    // Разрешить нажимать
    isWait=0;
    is_help_50=0;
    is_help_help=0;
    is_help_phone=0;

    // Скрыть табло
    tablo->hide();

}


void wGame::clickAnwser(int value){
    if (isWait==1) return;
    isWait=1;

    // Скрыть табло
    tablo->hide();

    // отключить фоновый звук (канал-1)
#ifndef __ANDROID__
    Mix_HaltChannel(1);
#endif

    // Отметить выбор и заблокировать кнопки
    for (int i=1;i<5;i++){
        anwser[i]->setPicEvent(":images/anwser_bg.png","","");
        anwser[i]->setBlock(1);
    }
    anwser[value]->setPicEvent(":images/anwser_hover_bg.png","","");

    // проиграть музыку ожидания
#ifndef __ANDROID__
    Mix_PlayMusic(sound_pause,0);
#endif

    // запомнить выбор
    anwserCheck=value;

    // перейти в режим ожидания ответа
    if (score<6) timer->start(1000);
    if (score>=6 and score<11) timer->start(3000);
    if (score>=11 and score<15) timer->start(4000);
    if (score==15) timer->start(6000);


}

void wGame::continueGame(){
    // продолжение игры (после паузы)
    timer->stop();


    anwser[anwserW]->setPicEvent(":images/anwser_right_bg.png","","");

    if (anwserCheck==anwserW){
#ifndef __ANDROID__
        Mix_PlayMusic(sound_true,0);
#endif
        score++;

        if (score==16){
#ifndef __ANDROID__
            Mix_PlayMusic(sound_win,0);
#endif
            tablo->setText("Игра окончена","Поздравляю, вы выиграли миллион рублей !");
            tablo->show();
            return;
        }

        if (score==6 or score==11){
#ifndef __ANDROID__
            Mix_PlayMusic(sound_start,0);
#endif
            timer_pause->start(11000);
            tablo->setText("Несгораемая сумма","Поздравляю, вы достигли несгораемой суммы !");
            tablo->show();
            return;
        }

        timer_pause->start(3000);

    }else{
#ifndef __ANDROID__
        Mix_PlayMusic(sound_false,0);
#endif
        tablo->setText("Игра окончена","К сожалению, вы неправильно ответили на вопрос.");
        tablo->show();
        return;
    }
}

void wGame::continueGame2(){
    //  Продолжение игры (после показа правильного ответа. Показ нового вопроса)
    timer_pause->stop();
    tablo->hide();

    // генерация нового вопроса
    genQuestion(score);

    for (int i=1;i<5;i++){
        anwser[i]->setPicEvent(":images/anwser_bg.png",":images/anwser_hover_bg.png","");
        anwser[i]->setBlock(0);
    }

    score_table->setScore(score);

    // запустить фоновую мелодию
#ifndef __ANDROID__
    if (score<6) Mix_PlayChannel(1,sound_fon[1],-1);
    if (score>=6 and score<11) Mix_PlayChannel(1,sound_fon[2],-1);
    if (score>=11) Mix_PlayChannel(1,sound_fon[3],-1);
#endif

    isWait=0;
}





void wGame::genQuestion(int level){
    QString sql;
    QString strNum;
    int nA=1,nB=1,nC=1,nD=1;    // номера мест в таблице ответов

    QSqlQuery query(QSqlDatabase::database());


    strNum.setNum(level,10);
    sql="SELECT question,a,b,c,d FROM question WHERE level="+strNum+" ORDER BY Random() LIMIT 1";
    if(!query.exec(sql)){
        QMessageBox::critical(this,"Error","Unable to establish a database connection1");
    }
    if (query.size()==0){
        QMessageBox::critical(this,"Error","Unable to establish a database connection2");
    }
    query.first();


    // перемешивание ответов
    nA=qrand()%4+1;
    anwserW=nA;
    answerWstr=query.value(1).toString();
    while (nB==nA) nB=qrand()%4+1;
    while (nC==nA or nC==nB) nC=qrand()%4+1;
    while (nD==nA or nD==nB or nD==nC) nD=qrand()%4+1;


    // Расстановка вопроса и ответов
    question->setText(query.value(0).toString());
    anwser[nA]->setText(query.value(1).toString());
    anwser[nB]->setText(query.value(2).toString());
    anwser[nC]->setText(query.value(3).toString());
    anwser[nD]->setText(query.value(4).toString());

    //qDebug() << anwserW;
}


void wGame::initButton(){
    // кнопки ответов
    anwser[1] = new wButton(this);
    anwser[1]->setPicEvent(":images/anwser_bg.png",":images/anwser_hover_bg.png","");
    anwser[1]->setText("  A:","Варинт 1");
    anwser[1]->setFixedSize(337,39);
    anwser[1]->move(10,400);

    anwser[2] = new wButton(this);
    anwser[2]->setPicEvent(":images/anwser_bg.png",":images/anwser_hover_bg.png","");
    anwser[2]->setText("  B:","Вариант 2");
    anwser[2]->setFixedSize(337,39);
    anwser[2]->move(10,445);

    anwser[3] = new wButton(this);
    anwser[3]->setPicEvent(":images/anwser_bg.png",":images/anwser_hover_bg.png","");
    anwser[3]->setText("  C:","Вариант 3");
    anwser[3]->setFixedSize(337,39);
    anwser[3]->move(364,400);

    anwser[4] = new wButton(this);
    anwser[4]->setPicEvent(":images/anwser_bg.png",":images/anwser_hover_bg.png","");
    anwser[4]->setText("  D:","Вариант 4");
    anwser[4]->setFixedSize(337,39);
    anwser[4]->move(364,445);


    // табло с вопросом
    question = new MButton(this);
    question->setPicEvent(":images/question_bg.png","");
    question->setText("Вопрос");
    question->setFixedSize(712,84);
    question->move(5,303);


    // кнопка закрыть
    btnClose = new MButton(this);
    btnClose->setPicEvent(":images/close.png",":images/close_hover.png");
    btnClose->setText("Закрыть");
    btnClose->setFixedSize(191,39);
    btnClose->move(750,460);


    // подсказки
    help_50 = new MButton(this);
    help_50->setPicEvent(":images/50.png",":images/50.png");
    help_50->setFixedSize(65,51);
    help_50->show();
    help_50->move(742,12);

    help_help = new MButton(this);
    help_help->setPicEvent(":images/help.png",":images/help.png");
    help_help->setFixedSize(65,51);
    help_help->show();
    help_help->move(815,12);

    help_phone = new MButton(this);
    help_phone->setPicEvent(":images/phone.png",":images/phone.png");
    help_phone->setFixedSize(65,51);
    help_phone->show();
    help_phone->move(888,12);


    // подключение сигналов
    signalMapper = new QSignalMapper(this);
    for (int i=1;i<5;i++){
        connect(anwser[i],SIGNAL(clicked()),signalMapper,SLOT(map()));
        signalMapper -> setMapping ( anwser[i], i);
    }
    connect (signalMapper, SIGNAL ( mapped ( int ) ), this, SLOT ( clickAnwser ( int ) ) );

    connect(help_50,SIGNAL(clicked()),this,SLOT(help50()));
    connect(help_help,SIGNAL(clicked()),this,SLOT(helpHelp()));
    connect(help_phone,SIGNAL(clicked()),this,SLOT(helpPhone()));
    connect(btnClose,SIGNAL(clicked()),this,SLOT(click_closeGames() ));


}

void wGame::click_closeGames(){
    // отключить фоновый звук (канал-1)
#ifndef __ANDROID__
    Mix_HaltChannel(1);
#endif

    emit closeGames();
}

void wGame::help50(){
    int n1=anwserW,n2=anwserW;
    if (is_help_50==1) return;

    while(n1==anwserW){
        n1=rand()%4+1;
    }
    while(n2==anwserW or n1==n2){
        n2=rand()%4+1;
    }
#ifndef __ANDROID__
    Mix_PlayMusic(sound_help,0);
#endif

    anwser[n1]->setText("");
    anwser[n2]->setText("");

    help_50->setPicEvent(":images/50_fail.png",":images/50_fail.png");
    is_help_50=1;
}

void wGame::helpHelp(){

    if (is_help_help==1) return;

    tablo->setText("Помощь зала","Большинство считают, что  "+answerWstr+" ");
    tablo->show();

#ifndef __ANDROID__
    Mix_PlayMusic(sound_help,0);
#endif

    help_help->setPicEvent(":images/help_fail.png",":images/help_fail.png");
    is_help_help=1;
}

void wGame::helpPhone(){
    if (is_help_phone==1) return;


    tablo->setText("Звонок другу","Ваш друг думает, что  "+answerWstr+" ");
    tablo->show();

#ifndef __ANDROID__
    Mix_PlayMusic(sound_help,0);
#endif

    help_phone->setPicEvent(":images/phone_fail.png",":images/phone_fail.png");
    is_help_phone=1;
}

void wGame::initSound(){
    QString strNum;
    QString gameDataPath;

#ifndef __ANDROID__
    if (QFile("sounds/start.wav").exists()){
        gameDataPath = "";
    }else if (QFile("/usr/share/million/sounds/start.wav").exists()){
        gameDataPath = "/usr/share/million/";
    }else{
        QMessageBox::critical(NULL,"Error","Unable to find game sound files");
        QApplication::exit();
    }

    // Загрузка звуковых файлов
    for (int i=1;i<4;i++){
        strNum.setNum(i,10);
        sound_fon[i] = Mix_LoadWAV(gameDataPath.toUtf8()+"sounds/fon"+strNum.toUtf8() +".wav");
        if(!sound_fon[i]) QMessageBox::warning(this,"No Sound",Mix_GetError());
    }

    sound_start = Mix_LoadMUS(gameDataPath.toUtf8()+"sounds/start.wav");
    if(!sound_start) QMessageBox::warning(this,"No Sound",Mix_GetError());

    sound_pause = Mix_LoadMUS(gameDataPath.toUtf8()+"sounds/pause.wav");
    if(!sound_pause) QMessageBox::warning(this,"No Sound",Mix_GetError());


    sound_true = Mix_LoadMUS(gameDataPath.toUtf8()+"sounds/true.wav");
    if(!sound_true) QMessageBox::warning(this,"No Sound",Mix_GetError());

    sound_false = Mix_LoadMUS(gameDataPath.toUtf8()+"sounds/false.wav");
    if(!sound_false) QMessageBox::warning(this,"No Sound",Mix_GetError());

    //sound_50 = Mix_LoadMUS(gameDataPath.toUtf8()+"sounds/50.wav");
    //if(!sound_50) QMessageBox::warning(this,"No Sound",Mix_GetError());

    sound_help = Mix_LoadMUS(gameDataPath.toUtf8()+"sounds/help.wav");
    if(!sound_help) QMessageBox::warning(this,"No Sound",Mix_GetError());

    //sound_phone = Mix_LoadMUS(gameDataPath.toUtf8()+"sounds/phone.wav");
    //if(!sound_phone) QMessageBox::warning(this,"No Sound",Mix_GetError());

    sound_win = Mix_LoadMUS(gameDataPath.toUtf8()+"sounds/win.wav");
    if(!sound_win) QMessageBox::warning(this,"No Sound",Mix_GetError());
#endif
}
